@@ -1,6 +1,7 @@
 import struct, socket, threading, json, os, pickle
 from essentials import tokening
 import essentials
+import copy
 
 def SocketDownload(sock):
     """
@@ -54,7 +55,7 @@ def ConnectorSocket(HOST, PORT):
     return  clientsocket
 
 class Socket_Server_Host:
-    def __init__(self, HOST, PORT, on_connection_open, on_data_recv, on_connection_close=False, daemon=True, autorun=True, connections=5):
+    def __init__(self, HOST, PORT, on_connection_open, on_data_recv, on_question, on_connection_close=False, daemon=True, autorun=True, connections=5):
         """Host your own Socket server to allows connections to this computer.
 
         Parameters
@@ -95,6 +96,7 @@ class Socket_Server_Host:
         self.HOST = HOST
         self.PORT = PORT
         self.connections = {}
+        self.on_question = on_question
         self.running = False
         if autorun:
             self.Run(connections, daemon)
@@ -130,7 +132,7 @@ class Socket_Server_Host:
             try:
                 conn, addr = self.server.accept()
                 conID = tokening.CreateToken(12, self.connections)
-                connector = Socket_Server_Client(conn, addr, conID, self.on_data_recv, on_close=self.close_connection)
+                connector = Socket_Server_Client(conn, addr, conID, self.on_data_recv, on_question=self.on_question, on_close=self.close_connection)
                 self.connections[conID] = connector
                 self.on_connection_open(connector)
             except Exception as e:
@@ -175,7 +177,7 @@ class Socket_Server_Host:
 
 class Socket_Server_Client:
 
-    def __init__(self, socket, addr, conID, on_data, on_close):
+    def __init__(self, socket, addr, conID, on_data, on_question, on_close):
         """Host your own Socket server to allows connections to this computer.
 
         Parameters
@@ -213,6 +215,8 @@ class Socket_Server_Client:
         self.on_close = on_close
         self.running = True
         self.meta = {}
+        self.on_question = on_question
+        self.__ask_list__ = {}
         self.created = essentials.TimeStamp()
         threading.Thread(target=self.__data_rev__, daemon=True).start()
 
@@ -248,6 +252,17 @@ class Socket_Server_Client:
         except:
             self.shutdown()
 
+    def ask(self, data):
+        tok = essentials.CreateToken(20, self.__ask_list__)
+        self.__ask_list__[tok] = False
+        self.send({"function_ask_question": tok, "data": data})
+        while self.__ask_list__[tok] == False:
+            pass
+        copyed = copy.deepcopy(self.__ask_list__[tok])
+        del self.__ask_list__[tok]
+        return copyed['data']
+
+
     def __data_rev__(self):
         """
         Server background task for accepting data and run the on_data event, you'll not need to use this.
@@ -261,11 +276,25 @@ class Socket_Server_Client:
             except:
                 self.shutdown()
                 return
-            self.on_data(data, self)
+            if type(data) == type({}) and 'function_ask_response' in data:
+                self.__ask_list__[data['function_ask_response']] = data
+            elif type(data) == type({}) and 'function_ask_question' in data:
+                self.on_question(Socket_Question(data['data'], self, data['function_ask_question']))
+            else:
+                self.on_data(data, self)
+
+class Socket_Question(object):
+    def __init__(self, data, client, tok):
+        self.data = data
+        self.questioner = client
+        self.__answer_token__ = tok
+    
+    def answer(self, data):
+        self.questioner.send({"function_ask_response": self.__answer_token__, "data": data})
 
 class Socket_Connector:
 
-    def __init__(self, HOST, PORT, on_data_recv, on_connection_close):
+    def __init__(self, HOST, PORT, on_data_recv, on_question, on_connection_close):
         """Host your own Socket server to allows connections to this computer.
 
         Parameters
@@ -276,6 +305,8 @@ class Socket_Connector:
 
         on_data_recv (:obj:`def`): The function to call when you receive data from a connection.
 
+        on_question (:obj:`def`): The function to call when you receive Socket_Question from a connection.
+
         on_connection_close (:obj:`def`, optional): The function to call when a connection is closed.
 
         Attributes
@@ -291,11 +322,23 @@ class Socket_Connector:
         self.running = True
         self.HOST = HOST
         self.PORT = PORT
+        self.__ask_list__ = {}
+        self.on_question = on_question
         self.on_connection_close = on_connection_close
         self.socket = ConnectorSocket(HOST, PORT)
         self.on_data_recv = on_data_recv
         threading.Thread(target=self.__data_rev__, daemon=True).start()
 
+    def ask(self, data):
+        tok = essentials.CreateToken(20, self.__ask_list__)
+        self.__ask_list__[tok] = False
+        self.send({"function_ask_question": tok, "data": data})
+        while self.__ask_list__[tok] == False:
+            pass
+        copyed = copy.deepcopy(self.__ask_list__[tok])
+        del self.__ask_list__[tok]
+        return copyed['data']
+
     def send(self, data):
         """
         Send data to the remote connection.
@@ -335,4 +378,9 @@ class Socket_Connector:
             except:
                 self.shutdown()
                 return
-            self.on_data_recv(data)
\ No newline at end of file
+            if type(data) == type({}) and 'function_ask_response' in data:
+                self.__ask_list__[data['function_ask_response']] = data
+            elif type(data) == type({}) and 'function_ask_question' in data:
+                self.on_question(Socket_Question(data['data'], self, data['function_ask_question']))
+            else:
+                self.on_data_recv(data)
\ No newline at end of file
